#
# Fit and remove 'red leak' continuum from a spectrum
#
# 2005-03-02 Bill Wyatt
#
# Requires 'hectospec' package for gettime task
# Requires 'images.imfit' package for fit1d task
#

procedure redleakcorr(input, output)

string input       = "" {prompt="input file"}
string output      = "" {prompt="output file"}
real   lambda      = 8520.0
int    band        = 1
int    endband     = 0     # -1 means do all bands in the image
bool   dounused    = no    # if 'yes' correct even unused bands
string fitfunc     = "spline3"
int    order       = 1
real   low_reject  = 3.0
real   high_reject = 2.0
int    niterate    = 20
bool   verbose     = no

begin

  string infile,   outfile
  string fitfile
  string section,  sect1,  sect2
  int    deltapix
  int    pixbeg,   pixend
  int    naxis
  int    naxis2
  string keyval
  int    crpix1
  real   cdelt1,   crval1
  real   lomean
  real   himean

  string history,  hdata

  naxis2  = 1
  if(band <= 0)  band = 1
  infile  = input
  outfile = output
  hdata   = "lambda "//str(lambda)//" fit "//fitfunc//" O("//str(order)//")"

 # get key values from header
  imgetp.verbose = no
  imgetp.default = "0"
  imgetp (infile, "i_naxis")
  if (imgetp.value == "0") {
    printf("redleakcorr: file %s: no NAXIS?\n", infile)
    error(1,"exiting...")
  }
  naxis = int (imgetp.value)

  imgetp.default = "0"
  imgetp (infile, "i_naxis1")
  if (imgetp.value == "0") {
    printf("redleakcorr: file %s: no NAXIS1?\n", infile)
    error(1,"exiting...")
  }
  pixend = int (imgetp.value)

  imgetp.default = "0"
  imgetp (infile, "i_naxis2")
  if (imgetp.value == "0") {
    printf("redleakcorr: file %s: band or endband >1 and no NAXIS2?\n", infile)
    error(1,"exiting...")
  }
  if(strlen(imgetp.value) > 0) 
    naxis2 = int (imgetp.value)

  if(endband >= 0 && endband < band)   
    endband = band  

  if(band > naxis2 && endband == band) {
    printf("redleakcorr: file %s: NAXIS2=%d, no band %d\n", infile, band)
    error(1,"exiting...")
  } else if(endband > naxis2) {
    printf("redleakcorr: file %s: NAXIS2=%d, bands= %d - %d\n", 
      infile, naxis2, band, endband)
    error(1,"exiting...")
  }
  if(endband < 0) endband = naxis2
  
  endband = min(endband, naxis2)

# Make sure redleak not done twice
  imgetp.default = "0"
  imgetp (infile, "REDLEAK")
  if (imgetp.value != "0") {
    printf("REDLEAK already done on %s\n", infile)
    return
  }

  imgetp.default = "0"
  imgetp (infile, "CRPIX1")
  if (imgetp.value == "0") {
    printf("redleakcorr: file %s: no CRPIX1?\n", infile)
    error(1,"exiting...")
  }
  crpix1 = int(imgetp.value)

  imgetp.default = "0"
  imgetp (infile, "CDELT1")
  if (imgetp.value == "0") {
    printf("redleakcorr: file %s: no CDELT1?\n", infile)
    error(1,"exiting...")
  }
  cdelt1 = real(imgetp.value)

  imgetp.default = "0"
  imgetp (infile, "CRVAL1")
  if (imgetp.value == "0") {
    printf("redleakcorr: file %s: no CRVAL1?\n", infile)
    error(1,"exiting...")
  }
  crval1 = real(imgetp.value)

  pixbeg = int((lambda - crval1)/cdelt1 + crpix1 - 1)

  if(verbose)
     printf("rdlc(%3d): lam= %.1f, sec [%d:%d], crpix= %d, crval= %.1f cdelt= %.3f\n",
       0, lambda, pixbeg, pixend, crpix1, crval1, cdelt1)

# most of output is a copy of the input
  imcopy(infile, outfile, verbose=no)

  fitfile  = osfn("tmp$") // mktemp("drdlk_") // ".fits"
  delete(fitfile,  ver-, >& "dev$null")

# Loop here for multiple bands
  for(i = band; i <= endband; i = i+1) {
  
    imgetp.default = "0"
    imgetp (infile, "APID" // str(i))
    keyval = imgetp.value
    if(! dounused && strlen(keyval) > 0 && substr(keyval,2,7) == "unused") {
      if(verbose) {
        printf("rdlc(%3d): %60s unused\n", i, " ")
        next
      }
    }
    section = "[" // str(pixbeg) // ":" // str(pixend) // "," // str(i) // "]"

    fit1d(infile // section,  fitfile,  "fit",  axis=1,
         inter-,                 sample="*",              naver=1,
         function=fitfunc,       order=order,             grow=0.,
         low_reject=low_reject,  high_reject=high_reject, niterate=niterate)
  
# If the continuum trends downwards there is no (significant) red leak and
# the subtraction should not be performed

    deltapix = (pixend - pixbeg) / 10
    if(deltapix < 3) {
      deltapix = min(3,pixend-pixbeg+1)
    }

# get mean of first 10% of fit
    sect1 = "["//str(pixbeg)//":"//str(pixbeg+deltapix-1) // "," //str(i)// "]"
    imstatistics(fitfile // sect1, 
            format=no, fields="mean", nclip=0, lower=INDEF, upper=INDEF) |
    scanf("%f", lomean)
  
# get mean of last 10% of fit
    sect2 = "["//str(pixend-deltapix+1)//":"//str(pixend)//","//str(i)// "]"
    imstatistics(fitfile // sect2,
            format=no, fields="mean", nclip=0, lower=INDEF, upper=INDEF) |
    scanf("%f", himean)

    if(verbose) {
       printf("rdlc(%3d): Fit mean%-15s= %6.1f  mean%-15s= %6.1f", 
              i, sect1, lomean, sect2, himean)
    }

    if(himean > lomean) {

      gettime(usesec=yes,verbose=no)

    # subtract off the fit
      imexpr("a - b + lov(b)",  outfile // section, 
         infile // section,  fitfile // section,  verbose-)
      if(verbose) printf(" corr")
    }
    if(verbose) printf("\n")

# End 'i=band' loop
  }

  delete(fitfile,  ver-, >& "dev$null")

  history = gettime.date//" "//gettime.time // " " // hdata
  hedit(outfile, "REDLEAK", history, add+, update+, show-, verify-)
  
end
