# XCPIXN - Piecewise cross-correlate one spectrum against another in pixel space
#         using MKXCTEMP and XCSAO
# March 8, 2006
# By Doug Mink, SAO Telescope Data Center

procedure xcpixn (object, template)

file	object		{prompt="Object spectrum file"}
file	template	{prompt="Template spectrum file"}

int	firstpix=150	{prompt="Number of first pixel to correlate"}
int	lastpix=4150	{prompt="Number of last pixel to correlate"}
int	npix=200	{prompt="Number of pixels to correlate"}
int	nbin=256	{prompt="Number of rebinned pixels to correlate"}
int	intpix=100	{prompt="Interval between correlation centers in pixels"}
real	refpix=INDEF	{prompt="Reference pixel in template and spectrum"}
real	shift=INDEF	{prompt="Pixel shift at reference pixel"}
real	stretch=INDEF	{prompt="Additional pixel shift per pixel"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
bool	interactive=no	{prompt="Fit dispersion function interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char arg
	file templist	# File to which to write list of segments
	file xcout	# File to which to write xcsao output
	file objfile, tempfile

	string s1, s2
	real r, y, d1, d2, d3, d4, x, cdelt1, y0
	real sx, sy, sxy, sx2, sm, a0, a1
	real stdev, tstdev, diff, x1, constant
	real rpix, nd
	int im

	if (refpix == INDEF) {
	    rpix = 1.0
	    }
	else {
	    rpix = refpix
	    }
	if (ndev < 1) {
	    nd = 10
	    }
	else {
	    nd = ndev
	    }

# If template file has negative values, put a pedestal under it
	tempfile = template
	istat (tempfile)
	if (istat.min < 0.0) {
	    constant = -istat.min + 100.0
	    imarith (tempfile, "+", constant, tempfile, verbose=debug)
	    }

# If object file has negative values, put a pedestal under it
	objfile = object
	istat (objfile)
	if (istat.min < 0.0) {
	    constant = -istat.min + 100.0
	    imarith (objfile, "+", constant, objfile, verbose=debug)
	    }

# Make list of templates npix wide every intpix pixels
	templist = mktemp ("xcpix")
	mkxctemp (tempfile,templist=templist,firstpix=firstpix,lastpix=lastpix,npix=npix,intpix=intpix)

# Cross-correlate spectrum against template, producing output in xcout
	rvsao (motd=no)
	arg = "@" // templist
	xcout = mktemp ("xcsao")
	xcsao (objfile,templates=arg,correlate="pixel",specnum=0,tempnum=0,ncols=nbin,report_mode=16,logfile=xcout)
	if (debug) {
	    print ("xcsao output written to ",xcout)
	    }
	delete (templist)

# Read pixels and shifts from xcout and find offset and stretch
	sx = 0.0
	sy = 0.0
	sxy = 0.0
	sx2 = 0.0
	sm = 0.0
	list = xcout
	while (fscan (list, s1, s2, r, y, d1, d2, d3, d4, x) != EOF) {
	    x1 = (x - rpix)
	    sx = sx + x1
	    sx2 = sx2 + (x1 * x1)
	    sxy = sxy + (x1 * y)
	    sy = sy + y
	    sm = sm + 1.0
	    if (debug) {
		print (sm," ", x1, " ", y)
		}
	    }
	a0 = ((sy * sx2) - (sx * sxy)) / ((sm * sx2) - (sx * sx))
	a1 = ((sm * sxy) - (sx * sy)) / ((sm * sx2) - (sx * sx))
	if (debug) {
	    if (a0 < 0) {
		printf ("x1 = x - %.6f + (x - %.1f) * %.8f\n", -a0, rpix, a1)
		}
	    else {
		printf ("x1 = x - %.6f + (x - %.1f) * %.8f\n",a0, rpix, a1)
		}
	    }

# Compute variance and standard deviation
	sy = 0.0
	list = xcout
	while (fscan (list, s1, s2, r, y, d1, d2, d3, d4, x) != EOF) {
	    x1 = (x - rpix)
	    y0 = a0 + (a1 * x1)
	    diff = y - y0
	    sy = sy + (diff * diff)
	    }
	stdev = sqrt (sy / sm)
	tstdev = ndev * stdev
	if (debug) {
	    print ("stdev=",stdev,", limit=",tstdev)
	    }

# Recompute offset and stretch, dropping points outside of ndev sigma
	sx = 0.0
	sy = 0.0
	sxy = 0.0
	sx2 = 0.0
	sm = 0.0
	im = 0
	list = xcout
	while (fscan (list, s1, s2, r, y, d1, d2, d3, d4, x) != EOF) {
	    x1 = (x - rpix)
	    y0 = a0 + (a1 * x1)
	    diff = y - y0
	    im = im + 1
	    if (diff < 0.0) {
		diff = y0 - y
		}
	    if (diff < tstdev) {
		sx = sx + x1
		sx2 = sx2 + (x1 * x1)
		sxy = sxy + (x1 * y)
		sy = sy + y
		sm = sm + 1.0
		}
	    else if (debug) {
		printf ("Drop %2d: %.2f -> %.2f (%.2f / %.2f)\n", im, x1, y,diff,tstdev)
		}
	    }
	shift = ((sy * sx2) - (sx * sxy)) / ((sm * sx2) - (sx * sx))
	stretch = ((sm * sxy) - (sx * sy)) / ((sm * sx2) - (sx * sx))
	if (debug) {
	    if (shift < 0) {
		printf ("x1 = x - %.6f + (x - %.1f) * %.8f\n", -shift, rpix, stretch)
		}
	    else {
		printf ("x1 = x - %.6f + (x - %.1f) * %.8f\n", shift, rpix, stretch)
		}
	    }
	print ("shift=", shift, "stretch=", stretch)
end

# Dec 18 2002	New task

# Mar  8 2006	Do not load saotdc and itools; istat is in hectospec
